class Competition(object):
  def __init__(self,
               name,
               parameters_description,
               goal,
               computation_budge,
               id=None,
               theoretical_best_metrics=None,
               current_best_metrics=None,
               status=None,
               created_time=None,
               updated_time=None):
    self.id = id
    self.name = name
    self.parameters_description = parameters_description
    self.goal = goal
    self.computation_budge = computation_budge
    self.theoretical_best_metrics = theoretical_best_metrics
    self.current_best_metrics = current_best_metrics
    self.status = status
    self.created_time = created_time
    self.updated_time = updated_time

  def __str__(self):
    return "{}_{}_{}".format(self.name, self.parameters_description, self.goal)

  def to_dict(self):
    return {
        "name": self.name,
        "parameters_description": self.parameters_description,
        "goal": self.goal,
        "computation_budge": self.computation_budge
    }

  @classmethod
  def create_from_dict(self, dict):
    return Competition(dict["name"], dict["parameters_description"],
                       dict["goal"], dict["computation_budge"], dict["id"],
                       dict["theoretical_best_metrics"],
                       dict["current_best_metrics"], dict["status"],
                       dict["created_time"], dict["updated_time"])


class Participation(object):
  def __init__(self,
               competition_id,
               username,
               email,
               id=None,
               current_best_metrics=None,
               current_trial_count=None,
               status=None,
               created_time=None,
               updated_time=None):
    self.id = id
    self.competition_id = competition_id
    self.username = username
    self.email = email
    self.current_best_metrics = current_best_metrics
    self.current_trial_count = current_trial_count
    self.status = status
    self.created_time = created_time
    self.updated_time = updated_time

  def __str__(self):
    return "{}_{}_{}".format(self.competition, self.username, self.email)

  def to_dict(self):
    return {
        "competition_id": self.competition_id,
        "username": self.username,
        "email": self.email
    }

  @classmethod
  def create_from_dict(self, dict):
    return Participation(
        dict["competition"]["id"], dict["username"], dict["email"], dict["id"],
        dict["current_best_metrics"], dict["current_trial_count"],
        dict["status"], dict["created_time"], dict["updated_time"])


class Trial(object):
  def __init__(self,
               participation_id,
               parameters_instance,
               id=None,
               metrics=None,
               status=None,
               created_time=None,
               updated_time=None):
    self.id = id
    self.participation_id = participation_id
    self.parameters_instance = parameters_instance
    self.metrics = metrics
    self.status = status
    self.created_time = created_time
    self.updated_time = updated_time

  def __str__(self):
    return "{}".format(self.parameters_instance)

  def to_dict(self):
    return {
        "participation_id": self.participation_id,
        "parameters_instance": self.parameters_instance
    }

  @classmethod
  def create_from_dict(self, dict):
    return Trial(dict["participation"]["id"], dict["parameters_instance"],
                 dict["id"], dict["metrics"], dict["status"],
                 dict["created_time"], dict["updated_time"])



# Copyright 2018.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

try:
  from setuptools import setup
  setup()
except ImportError:
  from distutils.core import setup

setup(
    name="tuninggame",
    version="0.1.0",
    author="tobe",
    author_email="tobeg3oogle@gmail.com",
    url="https://github.com/tobegit3hub/tuning_game",
    #install_requires=["requests>=2.6.0", "pyOpenSSL>=16.1.0",
    #                  "argcomplete>=1.4.1"],
    install_requires=[],
    description=
    "The sdk for the tuning game in https://github.com/tobegit3hub/tuning_game.",
    packages=["tuninggame"],
    entry_points={
        "console_scripts": [
            #"tuninggame=tuninggame.command:main"
        ],
    })



import json
import logging

import requests

from .models import Competition, Participation, Trial


class TuningGameClient(object):
  def __init__(self, endpoint="http://0.0.0.0:8000"):
    self.endpoint = endpoint

  def create_competition(self, name, parameters_description, goal,
                         computation_budge):
    url = "{}/tuning/v1/competitions".format(self.endpoint)
    request_data = {
        "name": name,
        "parameters_description": parameters_description,
        "goal": goal,
        "computation_budge": computation_budge
    }
    response = requests.post(url, json=request_data)

    competition = None
    if response.ok:
      competition = Competition.create_from_dict(response.json()["data"])

    return competition

  def list_competitions(self):
    url = "{}/tuning/v1/competitions".format(self.endpoint)
    response = requests.get(url)

    competitions = []
    if response.ok:
      dicts = response.json()["data"]
      for dict in dicts:
        competition = Competition.create_from_dict(dict)
        competitions.append(competition)

    return competitions

  def get_competition_by_id(self, competition_id):
    url = "{}/tuning/v1/competitions/{}".format(self.endpoint, competition_id)
    response = requests.get(url)

    competition = None
    if response.ok:
      competition = Competition.create_from_dict(response.json()["data"])

    return competition

  def get_competition_by_name(self, competition_name):
    url = "{}/tuning/v1/competitions/{}?name={}".format(
        self.endpoint, -1, competition_name)
    response = requests.get(url)

    competition = None
    if response.ok:
      competition = Competition.create_from_dict(response.json()["data"])

    return competition

  def delete_competition(self, competition_id):
    url = "{}/tuning/v1/competitions/{}".format(self.endpoint, competition_id)
    response = requests.delete(url)
    return response

  def create_participation(self, competition_id, username, email):
    url = "{}/tuning/v1/participations".format(self.endpoint)
    request_data = {
        "competition_id": competition_id,
        "username": username,
        "email": email
    }
    response = requests.post(url, json=request_data)

    participation = None
    if response.ok:
      participation = Participation.create_from_dict(response.json()["data"])

    return participation

  def list_participations(self):
    url = "{}/tuning/v1/participations".format(self.endpoint)
    response = requests.get(url)

    participations = []
    if response.ok:
      dicts = response.json()["data"]
      for dict in dicts:
        participation = Participation.create_from_dict(dict)
        participations.append(participation)

    return participations

  def get_participation_by_id(self, participation_id):
    url = "{}/tuning/v1/participations/{}".format(self.endpoint,
                                                  participation_id)
    response = requests.get(url)

    participation = None
    if response.ok:
      participation = Participation.create_from_dict(response.json()["data"])
    return participation

  def get_participation_by_competition_name_and_username(
      self, competition_name, participation_username):
    url = "{}/tuning/v1/participations/{}?competition_name={}&username={}".format(
        self.endpoint, -1, competition_name, participation_username)
    response = requests.get(url)

    participation = None
    if response.ok:
      participation = Participation.create_from_dict(response.json()["data"])
    return participation

    def get_participation_by_competition_name_and_email(
        self, competition_name, participation_email):
      url = "{}/tuning/v1/participations/{}?competition_name={}&email={}".format(
          self.endpoint, -1, competition_name, participation_email)

    response = requests.get(url)

    participation = None
    if response.ok:
      participation = Participation.create_from_dict(response.json()["data"])
    return participation

  def delete_participation(self, participation_id):
    url = "{}/tuning/v1/participations/{}".format(self.endpoint,
                                                  participation_id)
    response = requests.delete(url)
    return response

  def create_trial(self, participation_id, parameters_instance):
    url = "{}/tuning/v1/trials".format(self.endpoint)
    request_data = {
        "participation_id": participation_id,
        "parameters_instance": parameters_instance
    }
    response = requests.post(url, json=request_data)

    trial = None
    if response.ok:
      trial = Trial.create_from_dict(response.json()["data"])

    return trial

  def list_trials(self):
    url = "{}/tuning/v1/trials".format(self.endpoint)
    response = requests.get(url)

    trials = []
    if response.ok:
      dicts = response.json()["data"]
      for dict in dicts:
        trial = Trial.create_from_dict(dict)
        trials.append(trial)

    return trials

  def list_trials_by_participation_id(self, participation_id):
    url = "{}/tuning/v1/trials?participation_id={}".format(
        self.endpoint, participation_id)
    response = requests.get(url)

    trials = []
    if response.ok:
      dicts = response.json()["data"]
      for dict in dicts:
        trial = Trial.create_from_dict(dict)
        trials.append(trial)

    return trials

  def get_trial_by_id(self, trial_id):
    url = "{}/tuning/v1/trials/{}".format(self.endpoint, trial_id)
    response = requests.get(url)

    trial = None
    if response.ok:
      trial = Trial.create_from_dict(response.json()["data"])

    return trial

  def delete_trial(self, trial_id):
    url = "{}/tuning/v1/trials/{}".format(self.endpoint, trial_id)
    response = requests.delete(url)
    return response

  def execute_trial(self, trial_id):
    url = "{}/tuning/v1/trials/{}/execute".format(self.endpoint, trial_id)
    response = requests.post(url)
    return response

  def print_participation_result(self, participation_id):
    trials = self.list_trials_by_participation_id(participation_id)

    print("{:16} | {:32}".format("Metrics", "Parameters"))
    for trial in trials:
      print("{:16} | {:32}".format(trial.metrics, trial.parameters_instance))
